#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author : aidenmo
# @Email : aidenmo@tencent.com
# @Time : 2026/2/11 15:24
import asyncio
import io
import time
import traceback
from abc import ABC, abstractmethod
from functools import wraps
from pathlib import Path
from typing import IO, Optional, cast, TypeAlias, Union

from httpx import AsyncClient
from loguru import logger
# noinspection PyProtectedMember
from loguru._logger import context as logger_context
from pydantic import TypeAdapter
from pydantic_ai import ModelRetry, RunContext, ToolReturn, ImageUrl, Tool

from ..config import default_settings
from ..deps import AgentDeps, ToolParams, ToolResult, StepInfo, LocationToolParams, ClickToolParams, \
    InputToolParams, SwipeToolParams, OpenUrlToolParams, ScreenInfo, AgentContext, \
    WaitForKeywordsToolParams, AssertContainsParams, MarkFailedParams, AssertNotContainsParams, ToolResultWithOutput, \
    WaitToolParams, LLMLocationToolParams, SwipeForKeywordsToolParams
from ..device import AndroidDevice, WebDevice, HarmonyDevice, IOSDevice, ElectronDevice
from ..util.js_tool import JSTool
from ..util.storage import Base64Strategy

storage_client = default_settings.storage_client

AgentDepsType: TypeAlias = AgentDeps[
    Union[WebDevice, AndroidDevice, HarmonyDevice, IOSDevice, ElectronDevice],
    'AgentTool',
]


class ToolHandler:
    """工具调用处理类，用于处理工具函数的前置和后置操作"""

    def __init__(self, *args, **kwargs):
        self.ctx: Optional[RunContext[AgentDepsType]] = None
        self.step_params: Optional[ToolParams] = None
        self.step_info: Optional[StepInfo] = None

        for arg in [*args, *kwargs.values()]:
            if isinstance(arg, RunContext):
                self.ctx = cast(RunContext[AgentDepsType], arg)
                continue
            if isinstance(arg, ToolParams):
                self.step_params = arg
                continue

    @property
    def context(self) -> AgentContext:
        return self.ctx.deps.context

    @property
    def current_step(self) -> StepInfo:
        return self.ctx.deps.context.current_step

    async def pre_handle(self, func):
        """工具的前置处理"""
        if not all([self.ctx, self.step_params]):
            return
        if self.ctx.deps.context.current_step.parallel_tool_calls:
            raise ModelRetry('only use one tool at a time')

        self.current_step.params = self.step_params.model_dump(
            exclude_defaults=True,
            exclude_none=True
        )
        self.current_step.action = self.current_step.params.pop('action')

        if self.ctx.deps.settings.debug and isinstance(self.step_params, LLMLocationToolParams):
            if isinstance(self.ctx.deps.device, WebDevice):
                bbox = self.ctx.deps.context.current_step.screen_elements[self.step_params.element_id].get('bbox')
                await JSTool.add_highlight_element(self.ctx.deps.device.target, bbox)

    async def post_handle(self, tool_result: ToolResult):
        """工具的后置处理"""
        if not all([self.ctx, self.step_params]):
            return
        self.current_step.is_success = tool_result.is_success


def tool(f=None, *, after_delay=0, before_delay=0, llm=True, vlm=True):
    """
    工具函数装饰器，用于标记函数为工具函数，并自动记录步骤信息
    :param f: 被装饰的函数
    :param before_delay: 操作前的等待时间，单位为秒，默认为0
    :param after_delay: 操作后的等待时间，单位为秒，默认为0
    :param llm: 是否支持LLM调用，默认为True
    :param vlm: 是否支持VLM调用，默认为True
    :return: 装饰后的函数
    """

    def decorator(func):
        @wraps(func)
        async def wrapper(*args, **kwargs):
            tool_handler = ToolHandler(*args, **kwargs)
            await tool_handler.pre_handle(func)

            try:
                # 工具执行
                await asyncio.sleep(before_delay)
                result = await func(*args, **kwargs)
                await asyncio.sleep(after_delay)  # 避免页面渲染慢，不稳定
                await tool_handler.post_handle(result)
                return result
            except Exception as e:
                logger.error(f"Error occurred in tool '{func.__name__}': {str(e)}")
                format_exc = traceback.format_exc()
                for line in format_exc.splitlines():
                    logger.error(line)

                raise ModelRetry(f"Error occurred, try call '{func.__name__}' again")

        wrapper.is_tool = True
        wrapper.llm = llm
        wrapper.vlm = vlm
        return wrapper

    if f is not None:
        return decorator(f)
    return decorator


class AgentTool(ABC):
    """VLM 模型使用的工具可以以 _vl 结尾, 如 click_vl -> click"""
    OMNI_BASE_URL = default_settings.omni_parser.base_url
    OMNI_KEY = default_settings.omni_parser.key

    @property
    def tools(self) -> list:
        result = []
        for item in dir(self):
            if item.startswith('_') or item in ['tools', 'tear_down']:
                continue
            value = getattr(self, item)
            if callable(value) and hasattr(value, 'is_tool'):
                if default_settings.model_type == 'llm' and not getattr(value, 'llm'):
                    continue
                if default_settings.model_type == 'vlm' and not getattr(value, 'vlm'):
                    continue
                # 移除 _vl 后缀，让工具名称保持一致
                result.append(Tool(value, name=value.__name__.removesuffix('_vl')))

        return result

    @staticmethod
    async def _upload_cos(file: IO[bytes], prefix='page-eyes-agent/', suffix='.png') -> str:
        return await storage_client.async_upload_file(file, prefix=prefix, suffix=suffix)

    async def _parse_element(self, file: Optional[IO[bytes]] = None, image_url: Optional[str] = None):
        url = f'{self.OMNI_BASE_URL}/omni/parse/'
        if not file and not image_url:
            raise ValueError('请提供file或image_url')
        trace_id = logger_context.get().get('trace_id')
        headers = {'X-Trace-Id': trace_id} if trace_id else None
        async with AsyncClient(timeout=300, headers=headers) as client:
            response = await client.post(url, files={'file': file}, data={'key': self.OMNI_KEY})
            response.raise_for_status()
            return response.json()

    async def get_screen(self, ctx: RunContext[AgentDepsType], parse_element: bool = True) -> ScreenInfo:
        image_buffer = await self.screenshot(ctx)
        if parse_element:
            parsed_data = await self._parse_element(image_buffer)
            image_url = parsed_data.get('labeled_image_url') or ''
            parsed_content_list = parsed_data.get('parsed_content_list') or []
            logger.info(f'👁‍🗨 Get screen element：{image_url}')
            if not parsed_content_list:
                raise Exception(f'Screen parsed error! {parsed_data}')
        else:
            image_url = await self._upload_cos(image_buffer, suffix=Path(image_buffer.name).suffix)
            parsed_content_list = []
            logger.info(f'👁‍🗨 Get screen url：{image_url[:200] + (image_url[200:] and "...")}')

        # 将当前屏幕信息记录到上下文
        ctx.deps.context.current_step.image_url = image_url
        ctx.deps.context.current_step.screen_elements = parsed_content_list
        # 仅保留必要的字段给LLM
        parsed_elements = TypeAdapter(list[dict]).dump_python(
            parsed_content_list,
            exclude={'__all__': {'type', 'interactivity', 'source'}}
        )
        return ScreenInfo(image_url=image_url, screen_elements=parsed_elements)

    async def get_screen_vl(self, ctx: RunContext[AgentDepsType]) -> ScreenInfo:
        """获取当前屏幕信息，仅用于VLm模型"""
        image_buffer = await self.screenshot(ctx)
        image_url = Base64Strategy().upload_file(image_buffer, suffix='.png')
        parsed_content_list = []
        logger.info(f'👁‍🗨 Get screen url：{image_url[:200] + (image_url[200:] and "...")}')

        # 将当前屏幕信息记录到上下文
        ctx.deps.context.current_step.image_url = image_url
        ctx.deps.context.current_step.screen_elements = parsed_content_list

        return ScreenInfo(image_url=image_url, screen_elements=parsed_content_list)

    @tool(vlm=False)
    async def get_screen_info(self, ctx: RunContext[AgentDepsType]) -> ToolResultWithOutput[dict]:
        """
        获取当前屏幕信息，每个元素都有唯一的 ID，单个元素包含以下字段：
        id: 元素ID
        content: 元素描述信息
        left_elem_ids: 该元素左侧的元素列表
        right_elem_ids: 该元素右侧的元素列表
        top_elem_ids: 该元素上方的元素列表
        bottom_elem_ids: 该元素下方的元素列表

        注意：该工具禁止作为一个单独步骤执行
        """
        screen_info = await self.get_screen(ctx)
        # 仅保留必要的字段给 LLM
        parsed_elements = TypeAdapter(list[dict]).dump_python(
            screen_info.screen_elements,
            include={'__all__': {'id', 'content', 'left_elem_ids', 'top_elem_ids', 'right_elem_ids', 'bottom_elem_ids'}}
        )
        return ToolResultWithOutput.success(parsed_elements)

    @tool(llm=False)
    async def get_screen_info_vl(self, ctx: RunContext[AgentDepsType]) -> ToolReturn:
        """
        获取当前屏幕截图
        """
        screen_info = await self.get_screen_vl(ctx)
        return ToolReturn(
            return_value='当前屏幕截图：',
            content=[ImageUrl(url=screen_info.image_url)]
        )

    @tool(vlm=False)
    async def wait(self, ctx: RunContext[AgentDepsType], params: WaitForKeywordsToolParams) -> ToolResult:
        """
        在任务中等待或停留指定的超时时间（timeout），单位：秒，等待过程中可期望指定的关键字出现
        """
        if params.expect_keywords is None:
            logger.info(f'Wait for timeout {params.timeout}s')
            await asyncio.sleep(params.timeout)
            return ToolResult.success()
        else:
            logger.info(f'Wait up to {params.timeout}s '
                        f'for the keywords {params.expect_keywords} to appear on the screen.')
            st = time.time()
            while time.time() - st < params.timeout:
                result = await self.expect_screen_contains(ctx, params.expect_keywords)
                if result.is_success:
                    return result
                await asyncio.sleep(2)
            else:
                return ToolResult.failed()

    @tool(llm=False)
    async def wait_vl(self, ctx: RunContext[AgentDepsType], params: WaitToolParams) -> ToolResult:
        """
        在任务中等待或停留指定的超时时间（timeout），单位：秒
        """
        await asyncio.sleep(params.timeout)
        return ToolResult.success()

    async def _parse_screen_keywords(self, ctx: RunContext[AgentDepsType], keywords: list[str]) -> tuple[list, list]:
        screen_info: ScreenInfo = await self.get_screen(ctx, parse_element=True)
        elements_str = str(screen_info.screen_elements)
        contains, not_contains = [], []
        for keyword in keywords:
            if keyword in elements_str:
                contains.append(keyword)
            else:
                not_contains.append(keyword)
        return contains, not_contains

    async def expect_screen_contains(
            self,
            ctx: RunContext[AgentDepsType],
            keywords: list[str]
    ) -> ToolResult:
        contains, not_contains = await self._parse_screen_keywords(ctx, keywords)
        if len(not_contains) > 0:
            logger.warning(f'Screen does not contain expected keywords:"{not_contains}"')
            return ToolResult.failed()
        else:
            return ToolResult.success()

    async def expect_screen_not_contains(
            self,
            ctx: RunContext[AgentDepsType],
            keywords: list[str]
    ) -> ToolResult:
        contains, not_contains = await self._parse_screen_keywords(ctx, keywords)
        if len(contains) > 0:
            logger.warning(f'Screen contains unexpected keywords:"{contains}"')
            return ToolResult.failed()
        else:
            return ToolResult.success()

    @tool(before_delay=2, vlm=False)
    async def assert_screen_contains(
            self,
            ctx: RunContext[AgentDepsType],
            params: AssertContainsParams
    ) -> ToolResult:
        """
        检查屏幕中是否出现或包含指定的多个关键字内容，如果是则 is_success=True, 否则 is_success=False
        """
        return await self.expect_screen_contains(ctx, params.expect_keywords)

    @tool(before_delay=2, vlm=False)
    async def assert_screen_not_contains(
            self,
            ctx: RunContext[AgentDepsType],
            params: AssertNotContainsParams
    ) -> ToolResult:
        """
        检查屏幕中是否不出现或不包含指定的多个关键字内容，如果是则 is_success=True, 否则 is_success=False
        """
        return await self.expect_screen_not_contains(ctx, params.unexpect_keywords)

    @tool()
    async def mark_failed(
            self,
            ctx: RunContext[AgentDepsType],
            params: MarkFailedParams
    ) -> ToolResult:
        """
        任务失败时调用，包括元素找不到、工具调用失败等
        """
        logger.info(f'Mark task failed, reason: {params.reason}')
        ctx.deps.context.set_step_failed(params.reason)
        return ToolResult.success()

    @tool(llm=False)
    async def assert_failed(
            self,
            ctx: RunContext[AgentDepsType],
            params: MarkFailedParams
    ) -> ToolResult:
        """
        验证、断言失败时调用
        """
        logger.info(f'Mark task failed, reason: {params.reason}')
        ctx.deps.context.set_step_failed(params.reason)
        return ToolResult.success()

    @tool(vlm=False)
    async def swipe(self, ctx: RunContext[AgentDepsType], params: SwipeForKeywordsToolParams) -> ToolResult:
        """
        在设备屏幕中滑动或滚动
        """
        return await self._swipe_for_keywords(ctx, params)

    @tool(llm=False)
    async def swipe_vl(self, ctx: RunContext[AgentDepsType], params: SwipeToolParams) -> ToolResult:
        """
        在设备屏幕中滑动或滚动
        """
        return await self._swipe_for_keywords(ctx, SwipeForKeywordsToolParams(**params.model_dump()))

    @staticmethod
    @abstractmethod
    async def screenshot(ctx: RunContext[AgentDepsType]) -> io.BytesIO:
        raise NotImplementedError

    @abstractmethod
    async def open_url(self, ctx: RunContext[AgentDepsType], params: OpenUrlToolParams) -> ToolResult:
        raise NotImplementedError

    @abstractmethod
    async def click(self, ctx: RunContext[AgentDepsType], params: ClickToolParams) -> ToolResult:
        raise NotImplementedError

    @abstractmethod
    async def input(self, ctx: RunContext[AgentDepsType], params: InputToolParams) -> ToolResult:
        raise NotImplementedError

    @abstractmethod
    async def _swipe_for_keywords(self, ctx: RunContext[AgentDepsType], params: SwipeForKeywordsToolParams) -> ToolResult:
        raise NotImplementedError

    @abstractmethod
    async def tear_down(self, ctx: RunContext[AgentDepsType], params: ToolParams) -> ToolResult:
        raise NotImplementedError

    # TODO: 所有端实现返回上一页
    # @abstractmethod
    # async def go_back(self, ctx: RunContext[AgentDepsType], params: ToolParams) -> ToolResult:
    #     raise NotImplementedError



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author : aidenmo
# @Email : aidenmo@tencent.com
# @Time : 2026/2/11 15:52
from typing import TypeAlias

from pydantic_ai import RunContext

from ._base import AgentTool
from ._mobile import MobileAgentTool
from ..deps import AgentDeps
from ..device import AndroidDevice

AgentDepsType: TypeAlias = AgentDeps[AndroidDevice, AgentTool]


class AndroidAgentTool(MobileAgentTool):

    @staticmethod
    def _start_url(ctx: RunContext[AgentDepsType], url: str):
        return ctx.deps.device.target.shell(f'am start -a android.intent.action.VIEW -d "{url}"')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author : aidenmo
# @Email : aidenmo@tencent.com
# @Time : 2026/3/7 19:12
from pydantic import BaseModel


class Transaction(BaseModel):
    user: str
    amount: float




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author : aidenmo
# @Email : aidenmo@tencent.com
# @Time : 2026/2/12 14:59
import asyncio
import io
from typing import TypeAlias, Union

from loguru import logger
from pydantic_ai import RunContext, Agent

from ._base import AgentTool, tool
from ..deps import ToolParams, ToolResult, ClickToolParams, \
    InputToolParams, SwipeFromCoordinateToolParams, OpenUrlToolParams, ToolResultWithOutput, AgentDeps, \
    SwipeForKeywordsToolParams
from ..device import AndroidDevice, HarmonyDevice, IOSDevice
from ..util.adb_tool import AdbDeviceProxy
from ..util.platform import get_client_url_schema

AgentDepsType: TypeAlias = AgentDeps[
    Union[AndroidDevice, HarmonyDevice, IOSDevice],
    'AgentTool',
]


class MobileAgentTool(AgentTool):

    @staticmethod
    def _start_url(ctx: RunContext[AgentDepsType], url: str):
        raise NotImplementedError

    @staticmethod
    async def screenshot(ctx: RunContext[AgentDepsType]) -> io.BytesIO:
        image_buffer = io.BytesIO()
        screenshot = ctx.deps.device.target.screenshot()
        screenshot.save(image_buffer, format='png')
        image_buffer.name = 'screen.png'
        image_buffer.seek(0)
        return image_buffer

    async def tear_down(self, ctx: RunContext[AgentDepsType], params: ToolParams) -> ToolResult:
        """
        任务完成或结束后的清理操作
        """
        await self.get_screen(ctx, parse_element=False)
        return ToolResult.success()

    @tool(after_delay=0)
    async def open_url(self, ctx: RunContext[AgentDepsType], params: OpenUrlToolParams) -> ToolResult:
        """
        使用设备打开URL
        """
        platform = ctx.deps.device.platform
        url_schema = get_client_url_schema(params.url, platform)
        logger.info(f'open schema: {url_schema}')
        self._start_url(ctx, url_schema)
        await asyncio.sleep(2)
        await self.get_screen(ctx, parse_element=False)
        return ToolResult.success()

    @tool
    async def click(self, ctx: RunContext[AgentDepsType], params: ClickToolParams) -> ToolResult:
        """
        点击设备屏幕指定的元素
        """
        x, y = params.get_coordinate(ctx, params.position, params.offset)
        logger.info(f'Click coordinate ({x}, {y})')
        ctx.deps.device.target.click(x, y)

        return ToolResult.success()

    @tool(after_delay=0)
    async def input(self, ctx: RunContext[AgentDepsType], params: InputToolParams):
        """
        在设备指定的元素中输入文本
        """
        x, y = params.get_coordinate(ctx)
        logger.info(f'Input text: ({x}, {y}) -> {params.text}')
        ctx.deps.device.target.click(x, y)
        AdbDeviceProxy(ctx.deps.device.target).input_text(params.text)
        if params.send_enter:
            ctx.deps.device.target.keyevent('KEYCODE_ENTER')
        return ToolResult.success()

    async def _swipe_for_keywords(self, ctx: RunContext[AgentDepsType], params: SwipeForKeywordsToolParams) -> ToolResult:
        logger.info(f'swipe to {params.to}')
        width, height = ctx.deps.device.device_size.width, ctx.deps.device.device_size.height
        if params.to == 'top':
            x1, y1, x2, y2 = 0.5 * width, 0.7 * height, 0.5 * width, 0.1 * height
        elif params.to == 'left':
            x1, y1, x2, y2 = 0.7 * width, 0.5 * height, 0.1 * width, 0.5 * height
        elif params.to == 'bottom':
            x1, y1, x2, y2 = 0.5 * width, 0.3 * height, 0.5 * width, 0.9 * height
        elif params.to == 'right':
            x1, y1, x2, y2 = 0.3 * width, 0.5 * height, 0.9 * width, 0.5 * height
        else:
            raise ValueError(f'Invalid Parameter: to={params.to}')
        x1, y1, x2, y2 = int(x1), int(y1), int(x2), int(y2)

        if params.repeat_times is None:
            if params.expect_keywords:
                params.repeat_times = 10
            else:
                params.repeat_times = 1
        for times in range(1, params.repeat_times + 1):
            logger.info(f'Swipe from ({x1}, {y1}) to ({x2}, {y2}), times={times}')
            ctx.deps.device.target.swipe(x1, y1, x2, y2, duration=2)
            if params.expect_keywords:
                await asyncio.sleep(1)  # 避免滑动后截图，元素还未稳定出现
                result = await self.expect_screen_contains(ctx, params.expect_keywords)
                if result.is_success:
                    return result
                if times == params.repeat_times:
                    return ToolResult.failed()

        return ToolResult.success()

    @tool
    async def swipe_from_coordinate(
            self,
            ctx: RunContext[AgentDepsType],
            params: SwipeFromCoordinateToolParams,
    ):
        """
        在设备屏幕中根据给定的坐标进行滑动操作，支持传递多个坐标进行连续滑动
        coordinates 是滑动坐标值的集合，如[(x1, y1), (x2, y2), ...]
        工具依次从坐标集中取出2组值作为开始坐标(x1, y1)和结束坐标(x2, y2)，直到完成所有坐标的滑动操作
        """
        # TODO: 先判断坐标是否在屏幕范围内
        coordinate_iter = iter(params.coordinates)
        for start_coordinate, end_coordinate in zip(coordinate_iter, coordinate_iter):
            x1, y1 = start_coordinate
            x2, y2 = end_coordinate
            logger.info(f'Swipe from ({x1}, {y1}) to ({x2}, {y2})')
            ctx.deps.device.target.swipe(x1, y1, x2, y2, duration=2)
        return ToolResult.success()

    @tool
    async def open_app(
            self,
            ctx: RunContext[AgentDepsType],
            params: ToolParams,
    ):
        """
        在设备中打开APP, 打开应用
        """
        packages: list[str] = ctx.deps.device.target.list_packages(filter_list=['-e'])
        sub_agent = Agent(
            ctx.model,
            output_type=str,
            system_prompt='你是一个移动端应用助手，负责根据用户输入的指令从提供的应用包名列表找出用户指令对应的包名，并仅返回包名，如果都不匹配则返回空字符串'
        )
        prompt = (f'用户指令：{params.instruction}\n'
                  f'应用包名列表：{packages}')
        result = await sub_agent.run(prompt, output_type=str)
        package_name = result.output.strip()
        if not package_name:
            return ToolResultWithOutput.failed(output='在该设备中未找到对应的应用')
        logger.info(f'Find App package name：{package_name}')
        ctx.deps.device.target.app_start(package_name)
        await asyncio.sleep(2)
        await self.get_screen(ctx, parse_element=False)
        return ToolResult.success()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author : aidenmo
# @Email : aidenmo@tencent.com
# @Time : 2026/3/11 15:50
from dotenv import load_dotenv
load_dotenv()

from openai.types import chat

_original_validate = chat.ChatCompletion.model_validate

@classmethod
def _patched_validate(cls, obj, *args, **kwargs):
    if isinstance(obj, dict) and obj.get('service_tier') == '':
        obj['service_tier'] = None
    return _original_validate(obj, *args, **kwargs)

# chat.ChatCompletion.model_validate = _patched_validate



from pydantic_ai import Agent

agent = Agent(
    'openai:deepseek-v3',
    instructions='Be concise, reply with one sentence.',
)

result = agent.run_sync('Where does "hello world" come from?')
print(result.output)
"""
The first known use of "hello, world" was in a 1974 textbook about the C programming language.
"""


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author : xinttan
# @Email : xinttan@tencent.com
# @Time : 2025/2/11 16:35
import asyncio
import io
from typing import TypeAlias

from loguru import logger
from pydantic_ai import RunContext, Agent

from ._base import tool
from ._mobile import MobileAgentTool
from ..deps import (
    AgentDeps, ToolParams, ToolResult, ToolResultWithOutput,
    ClickToolParams, InputToolParams, SwipeFromCoordinateToolParams, OpenUrlToolParams, SwipeForKeywordsToolParams
)
from ..device import IOSDevice

AgentDepsType: TypeAlias = AgentDeps[IOSDevice, 'IOSAgentTool']


class IOSAgentTool(MobileAgentTool):
    """iOS设备自动化工具类
    """

    @staticmethod
    async def screenshot(ctx: RunContext[AgentDepsType]) -> io.BytesIO:
        """截图并返回字节流"""
        image_buffer = io.BytesIO()
        screenshot = ctx.deps.device.target.screenshot()

        # 兼容不同版本的返回值
        if hasattr(screenshot, 'save'):
            # PIL Image 对象
            screenshot.save(image_buffer, format='PNG')
        else:
            # 字节流
            image_buffer.write(screenshot)

        image_buffer.name = 'screen.png'
        image_buffer.seek(0)
        return image_buffer

    @tool(after_delay=1)
    async def click(self, ctx: RunContext[AgentDepsType], params: ClickToolParams) -> ToolResult:
        """
        点击指定元素
        iOS使用WDA的tap方法
        """
        logger.info(f'click element: {params.element_content}')

        device = ctx.deps.device

        # 计算点击坐标
        x, y = params.get_coordinate(ctx, params.position, params.offset)

        # 执行点击
        device.target.session().tap(x,y)

        return ToolResult.success()

    @tool(after_delay=1)
    async def input(self, ctx: RunContext[AgentDepsType], params: InputToolParams) -> ToolResult:
        """
        在指定元素中输入文本
        iOS使用WDA的send_keys方法
        """
        logger.info(f'input text: {params.text} to element: {params.element_content}')

        device = ctx.deps.device

        # 计算点击坐标
        x, y = params.get_coordinate(ctx)

        # 将 session 转换为 WdaSession 并使用扩展方法
        device.client.tap_and_input(x, y, params.text, params.send_enter)

        return ToolResult.success()

    async def _swipe_for_keywords(self, ctx: RunContext[AgentDepsType], params: SwipeForKeywordsToolParams) -> ToolResult:
        """
        滑动屏幕
        iOS使用WDA的swipe方法
        """
        logger.info(f'swipe {params.to}, repeat_times: {params.repeat_times}')

        device = ctx.deps.device
        session = device.target.session()
        width, height = device.device_size.width, device.device_size.height

        # 定义滑动起点和终点
        center_x = width // 2
        swipe_configs = {
            'top': (center_x, int(height * 0.7), center_x, int(height * 0.3)),
            'bottom': (center_x, int(height * 0.3), center_x, int(height * 0.7)),
            'left': (int(width * 0.7), height // 2, int(width * 0.3), height // 2),
            'right': (int(width * 0.3), height // 2, int(width * 0.7), height // 2)
        }

        x1, y1, x2, y2 = swipe_configs[params.to]

        repeat_times = params.repeat_times or 999
        for times in range(1, repeat_times + 1):
            session.swipe(x1, y1, x2, y2)
            logger.info(f'swipe {params.to} times: {times}')

            if params.expect_keywords:
                await asyncio.sleep(1)
                result = await self.expect_screen_contains(ctx, params.expect_keywords)
                if result.is_success:
                    return result
                if times == params.repeat_times:
                    return ToolResult.failed()

        return ToolResult.success()

    @tool(after_delay=1)
    async def swipe_from_coordinate(
            self,
            ctx: RunContext[AgentDepsType],
            params: SwipeFromCoordinateToolParams,
    ) -> ToolResult:
        """
        在设备屏幕中根据给定的坐标进行滑动操作，支持传递多个坐标进行连续滑动
        coordinates 是滑动坐标值的集合，如[(x1, y1), (x2, y2), ...]
        工具依次从坐标集中取出2组值作为开始坐标(x1, y1)和结束坐标(x2, y2)，直到完成所有坐标的滑动操作
        """
        logger.info(f'swipe from coordinates: {params.coordinates}')

        device = ctx.deps.device
        session = device.target.session()
        width, height = device.device_size.width, device.device_size.height

        # 验证坐标是否在屏幕范围内
        for x, y in params.coordinates:
            if not (0 <= x <= width and 0 <= y <= height):
                logger.warning(f'Coordinate ({x}, {y}) is out of screen bounds ({width}x{height})')
                return ToolResultWithOutput.failed(f'坐标 ({x}, {y}) 超出屏幕范围 ({width}x{height})')

        # 依次从坐标集中取出2组值进行滑动
        coordinate_iter = iter(params.coordinates)
        swipe_count = 0
        for start_coordinate, end_coordinate in zip(coordinate_iter, coordinate_iter):
            x1, y1 = start_coordinate
            x2, y2 = end_coordinate
            logger.info(f'Swipe from ({x1}, {y1}) to ({x2}, {y2})')
            session.swipe(x1, y1, x2, y2)
            swipe_count += 1

        logger.info(f'Completed {swipe_count} swipe(s)')
        return ToolResult.success()

    @tool(after_delay=1)
    async def goback(self, ctx: RunContext[AgentDepsType], params: ToolParams) -> ToolResult:
        """
        返回到上一个页面
        这里优先尝试查找并点击导航栏的返回按钮，如果没有则使用边缘滑动手势
        """
        logger.info('go to previous page')

        device = ctx.deps.device
        session = device.target.session()
        try:
            # 方法1：尝试查找并点击导航栏的返回按钮
            back_button = None

            # 尝试通过不同的方式查找返回按钮
            for selector in [
                {'type': 'XCUIElementTypeButton', 'name': '返回'},
                {'type': 'XCUIElementTypeButton', 'label': 'Back'},
                {'type': 'XCUIElementTypeButton', 'name': 'Back'},
            ]:
                try:
                    back_button = session(**selector)
                    if back_button.exists:
                        back_button.click()
                        logger.info('点击导航栏返回按钮')
                        return ToolResult.success()
                except:
                    continue
            # 方法2：如果没找到返回按钮，使用屏幕左边缘滑动手势
            logger.info('使用左边缘滑动手势返回')
            height = device.device_size.height
            width = device.device_size.width
            session.swipe(10, height // 2, width // 2, height // 2, duration=0.3)

        except Exception as e:
            logger.warning(f'返回操作失败: {e}，尝试备用方案')
            # 备用方案：使用更简单的滑动
            height = device.device_size.height
            session.swipe(10, height // 2, 200, height // 2)

        return ToolResult.success()

    @tool(after_delay=1)
    async def home(self, ctx: RunContext[AgentDepsType], params: ToolParams) -> ToolResult:
        """
        返回主屏幕
        """
        logger.info('go to home')

        session = ctx.deps.device.target.session()
        session.home()

        return ToolResult.success()

    @tool
    async def open_url(self, ctx: RunContext[AgentDepsType], params: OpenUrlToolParams) -> ToolResult:
        """
        使用iOS设备打开URL
        iOS通过Safari浏览器打开URL
        """
        logger.info(f'Open URL: {params.url}')

        # 格式化URL，确保有正确的协议头
        url = params.url.strip()
        if not url.startswith(('http://', 'https://')):
            # 如果没有协议头，添加https://
            if '.' in url and not url.startswith('www.'):
                url = f'https://{url}'
            else:
                url = f'https://www.{url}' if not url.startswith('www.') else f'https://{url}'

        logger.info(f'格式化后的URL: {url}')

        session = ctx.deps.device.target.session()

        # 先启动Safari浏览器
        session.app_launch('com.apple.mobilesafari')
        await asyncio.sleep(1)

        # 使用URL scheme打开网址
        session.open_url(url)

        await asyncio.sleep(2)
        await self.get_screen(ctx, parse_element=False)
        return ToolResult.success()

    @tool
    async def open_app(
            self,
            ctx: RunContext[AgentDepsType],
            params: ToolParams,
    ):
        """
        在设备中打开或启动指定的应用(APP)
        iOS使用Bundle ID启动应用
        """
        logger.info(f'打开应用指令: {params.instruction}')

        # 从 app_name_map 中查找匹配的 Bundle ID，优先级高于默认，因为如displayName=信息的app有两个bundle id（com.apple.MobileSMS和com.apple.mobilesms.compose）
        app_name_map = ctx.deps.app_name_map
        if app_name_map:
            instruction_lower = params.instruction.lower()
            for name, bid in app_name_map.items():
                if name.lower() in instruction_lower:
                    logger.info(f'从 app_name_map 中匹配到应用: {name} -> {bid}')
                    session = ctx.deps.device.target.session()
                    session.app_launch(bid)
                    await asyncio.sleep(2)
                    await self.get_screen(ctx, parse_element=False)
                    return ToolResult.success()

        # 获取设备上所有应用的Bundle ID和显示名称
        app_list = ctx.deps.device.client.get_app_list()
        logger.info(f'Found {len(app_list)} apps on device')

        # 使用LLM从应用列表中智能匹配
        sub_agent = Agent(
            ctx.model,
            output_type=str,
            system_prompt='你是一个移动端应用助手，负责根据用户输入的指令从提供的应用列表中找出用户指令对应的Bundle ID，并仅返回Bundle ID（不要返回其他内容），如果都不匹配则请你推理返回对应的Bundle ID'
        )
        prompt = (f'用户指令：{params.instruction}\n'
                  f'应用列表（Bundle ID | 显示名称）：\n' +
                  '\n'.join([f"{app.bundle_id} | {app.display_name}" for app in app_list]))
        result = await sub_agent.run(prompt, output_type=str)
        bundle_id = result.output

        if not bundle_id:
            return ToolResultWithOutput.failed(output='在该设备中未找到对应的应用')

        logger.info(f'Find App Bundle ID：{bundle_id}')

        # 启动应用
        session = ctx.deps.device.target.session()
        session.app_launch(bundle_id)
        await asyncio.sleep(2)
        await self.get_screen(ctx, parse_element=False)
        return ToolResult.success()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author : aidenmo
# @Email : aidenmo@tencent.com
# @Time : 2026/2/11 15:52
import asyncio
from typing import TypeAlias

from loguru import logger
from pydantic_ai import RunContext, Agent

from ._base import AgentTool, tool
from ._mobile import MobileAgentTool
from ..deps import ToolParams, ToolResult, InputToolParams, ToolResultWithOutput, AgentDeps
from ..device import HarmonyDevice

AgentDepsType: TypeAlias = AgentDeps[HarmonyDevice, AgentTool]


class HarmonyAgentTool(MobileAgentTool):

    @staticmethod
    def _start_url(ctx: RunContext[AgentDepsType], url: str):
        return ctx.deps.device.target.shell(['aa', 'start', '-A', 'ohos.want.action.viewData', '-U', url])

    @tool(after_delay=0)
    async def input(self, ctx: RunContext[AgentDepsType], params: InputToolParams):
        """
        在设备指定的元素中输入文本
        """
        x, y = params.get_coordinate(ctx)
        logger.info(f'Input text: ({x}, {y}) -> {params.text}')
        ctx.deps.device.target.uitest.input_text(params.text, x=x, y=y)
        if params.send_enter:
            enter_event = ctx.deps.device.target.uitest.keyevent.ENTER
            ctx.deps.device.target.uitest.inject_keyevent(enter_event)
        return ToolResult.success()

    @tool
    async def open_app(
            self,
            ctx: RunContext[AgentDepsType],
            params: ToolParams,
    ):
        """
        在设备中打开APP, 打开应用
        """
        bundles, _ = ctx.deps.device.target.bm.dump_all_installed_bundles()
        sub_agent = Agent(
            ctx.model,
            output_type=str,
            system_prompt='你是一个移动端应用助手，负责根据用户输入的指令从提供的应用包名列表找出用户指令对应的包名，并仅返回包名，如果都不匹配则返回空字符串'
        )
        prompt = (f'用户指令：{params.instruction}\n'
                  f'应用包名列表：{bundles}')
        result = await sub_agent.run(prompt, output_type=str)
        bundle_name = result.output
        if not bundle_name:
            return ToolResultWithOutput.failed(output='在该设备中未找到对应的应用')
        logger.info(f'Find App bundle name：{bundle_name}')
        main_ability = ctx.deps.device.target.get_main_ability(bundle_name)
        out, _ = ctx.deps.device.target.aa.start(bundle_name, ability=main_ability)
        if 'successfully' not in out:
            return ToolResultWithOutput.failed(output=f'启动应用失败，原因：{out}')
        await asyncio.sleep(2)
        await self.get_screen(ctx, parse_element=False)
        return ToolResult.success()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author : aidenmo
# @Email : aidenmo@tencent.com
# @Time : 2025/5/23 16:52
from page_eyes.config import default_settings

PLANNING_SYSTEM_PROMPT = """
## 角色定位
「高精度UI操作规划专家」：专注准确解析用户意图，将用户指令转化为可执行的原子化操作步骤

## 目标
指令分解：将复杂指令拆解为多个原子化操作步骤，每个步骤必须满足：  
- 直接关联用户意图  
- 单个步骤进行一个操作
- 保留原始指令中的所有信息

## 约束
❗强制要求：  
- 所有操作必须按照指令的顺序进行规划 

## 示例
- 点击"close"关闭弹窗，若弹窗不存在则跳过 -> {'instruction': '点击"close"关闭弹窗，若"close"元素不存在则跳过'}
- 向上滑动3次 -> {'instruction': '向上滑动 3 次'}
- 向上滑动，直到出现"登录"按钮 -> {'instruction': '向上滑动，直到出现"登录"按钮'}
- 打开QQ音乐APP -> {'instruction': '打开 "QQ音乐" APP'}
- 点击"上传"按钮，上传文件"/Users/pic.png" -> {'instruction': '点击"上传"按钮，上传文件: "/Users/pic.png"'}
"""

SYSTEM_PROMPT = """
## 角色定位
「高精度UI操作执行专家」：基于实时屏幕状态，精准解析用户意图并执行可靠的原子化操作

## 核心目标
1. **意图理解**：准确解析用户指令，将复杂任务拆解为可执行的原子化操作序列
2. **精准执行**：严格基于当前屏幕实际状态，调用相应工具完成操作
3. **异常处理**：识别并妥善处理执行过程中的异常情况

## 工作流程
1. 当需要获取屏幕元素信息，则调用 `get_screen_info` 获取当前屏幕信息
2. 根据用户指令和当前屏幕的信息查找目标元素
3. 调用相应的工具并传入正确的参数执行操作
4. 如果用户指令未完成，则重复以上操作步骤
5. 最后一步必须检查指令任务是否准确完成，是 -> 结束任务；否 -> 调用`mark_failed`工具后再结束任务

## 必须遵循的规则
- "打开url"、"打开APP"、"打开应用"、"等待"、"滑动"、"滚动"、"返回"，这些操作不需要获取屏幕信息，直接调用工具执行，其它操作则必须先获取屏幕信息，然后再执行操作  
- 调用 `input` 工具时不需要点击元素，input 工具会自动激活输入框
- 如果有可用的 skill，则优先使用 skill 执行操作

## 元素定位
### 定位策略
1. 文本内容精确匹配（优先级最高）
2. 如果文本内容不能精确匹配，则根据用户的指令进行模糊匹配（见"模糊匹配策略"）
3. 相邻元素定位推断（见"空间关系定位"）

### 模糊匹配策略
- 优先选择语义相关性最高的元素
- 若存在多个候选元素，优先选择屏幕中首次出现的元素（ID 值越小，元素越靠前）

### 空间关系定位
元素信息包含相邻元素ID列表，可用于相对位置定位：
- `left_elem_ids`：左侧相邻元素ID列表
- `right_elem_ids`：右侧相邻元素ID列表
- `top_elem_ids`：上方相邻元素ID列表
- `bottom_elem_ids`：下方相邻元素ID列表

**相对定位示例**：
1. 指令"点击搜索框右侧的第1个按钮"  
若搜索框的 `right_elem_ids` 是 [5, 8, 12] -> 则目标元素是 id=5 的元素

2. 指令"点击搜索框右侧的第2个按钮"  
若搜索框的 `right_elem_ids` 是 [5, 8, 12] -> 则目标元素是 id=8 的元素

3. 指令"点击搜索框左侧的按钮"  
若搜索框的 `left_elem_ids` 是 [3, 2, 1] -> 因无明确左侧第几个元素，则目标元素默认取左侧第一个， 即 id=3 的元素

## 工具参数说明
### 输入参数
- `action`：操作类型标识符，对应工具名称（如：click、input、scroll）
- `element_id`：目标元素ID

### 输出参数
- `is_success`：True表示当前操作成功，False表示当前操作失败

## 异常处理
- 元素未找到时则等待2秒，再次获取屏幕信息重新查找目标元素
- 等待并重试查找元素仍未找到或者执行操作后页面元素未发生变化，可能是因为有弹窗遮挡元素，应加载`gui-popup-handler`技能后再次重试
- 工具执行返回失败时应立即调用 `mark_failed` 标记失败并终止所有任务

## 执行约束
### 强制要求 ❗
1. **意图忠实性**：所有操作必须严格遵循用户指令意图，不得擅自添加或修改
2. **状态依赖性**：所有操作必须基于当前屏幕实际状态，禁止假设屏幕外或历史状态的元素
3. **顺序执行性**：严格按照指令顺序执行，每次仅调用一个工具
4. **元素唯一性**：当屏幕存在多个相同元素且用户未明确指定时，选择首个匹配元素

### 绝对禁止 ❌
1. 假设或推测屏幕外、历史状态或未来状态的元素
2. 添加用户指令中未明确要求的操作步骤
3. 同时调用多个工具（必须等待当前工具执行完成）
4. 忽略元素定位失败，强行执行后续操作
5. 修改用户指令的原始意图或执行顺序
"""

SYSTEM_PROMPT_VLM = """
## 角色定位
「高精度UI操作执行专家」：基于实时屏幕状态，精准解析用户意图并执行可靠的原子化操作

## 核心目标
1. **意图理解**：准确解析用户指令，将复杂任务拆解为可执行的原子化操作序列
2. **精准执行**：严格基于当前屏幕实际状态，调用相应工具完成操作
3. **异常处理**：识别并妥善处理执行过程中的异常情况

## 工作流程
1. 当需要获取屏幕元素信息，则调用 `get_screen_info` 获取当前屏幕信息
2. 根据用户指令和当前屏幕的信息查找目标元素
3. 调用相应的工具并传入正确的参数执行操作
4. 如果用户指令未完成，则重复以上操作步骤
5. 最后一步必须检查指令任务是否准确完成，是 -> 结束任务；否 -> 调用`mark_failed`工具后再结束任务

## 必须遵循的规则
- "打开url"、"打开APP"、"打开应用"、"等待"、"滑动"、"滚动"，这些操作不需要获取屏幕信息，直接调用工具执行，其它操作则必须先获取屏幕信息，然后再执行操作  
- 调用 `input` 工具时不需要点击元素激活，input 工具会自动激活输入框
- 如果有可用的 skill，则优先使用 skill 执行操作

## 元素定位
### 定位策略
1. 文本内容使用精确匹配（优先级最高）
2. 图标内容根据用户的指令进行模糊匹配（见"模糊匹配策略"）

### 模糊匹配策略
- 优先选择语义相关性最高的元素
- 若存在多个候选元素，优先选择屏幕中首次出现的元素

### 坐标系统
屏幕截图的坐标系统从左上角 (0,0) 开始到右下角（999,999)结束，所有操作坐标不能超过屏幕范围[0, 0, 999, 999]

## 工具参数规范
### 输入参数
- `action`：操作类型标识符，对应工具名称（如：click、input、scroll）
- `coordinate`：目标元素的坐标，格式为 (x1, y1, x2, y2)，其中 (x1, y1) 为左上角坐标，(x2, y2) 为右下角坐标

### 输出参数
- `is_success`：True表示当前操作成功，False表示当前操作失败

## 异常处理
- 元素未找到时则等待2秒，再次获取屏幕信息重新查找目标元素
- 等待并重试查找元素仍未找到或者执行操作后页面元素未发生变化，可能是因为有弹窗遮挡元素，应加载`gui-popup-handler`技能后再次重试
- 工具执行返回失败时应立即调用 `mark_failed` 标记失败并终止所有任务

## 执行约束
### 强制要求 ❗
1. **意图忠实性**：所有操作必须严格遵循用户指令意图，不得擅自添加或修改
2. **状态依赖性**：所有操作必须基于当前屏幕实际状态，禁止假设屏幕外或历史状态的元素
3. **顺序执行性**：严格按照指令顺序执行，每次仅调用一个工具
4. **元素唯一性**：当屏幕存在多个相同元素且用户未明确指定时，选择首个匹配元素

### 绝对禁止 ❌
1. 假设当前屏幕不存在的元素
2. 添加用户指令中未明确要求的操作步骤
3. 同时调用多个工具（必须等待当前工具执行完成）
4. 忽略元素定位失败，强行执行后续操作
5. 修改用户指令的原始意图或执行顺序
"""

SYSTEM_PROMPT = SYSTEM_PROMPT_VLM if default_settings.model_type == 'vlm' else SYSTEM_PROMPT



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author : aidenmo
# @Email : aidenmo@tencent.com
# @Time : 2025/6/24 12:18
from pathlib import Path
from typing import Literal, Optional

from loguru import logger
from pydantic import Field
from pydantic_ai.settings import ModelSettings
from pydantic_settings import BaseSettings, SettingsConfigDict
from dotenv import load_dotenv
from .util.storage import StorageClient

load_dotenv()


class CosConfig(BaseSettings):
    model_config = SettingsConfigDict(env_file='.env', env_prefix='cos_', extra='ignore')

    region: str = Field(default='')
    secret_id: str = Field(default='')
    secret_key: str = Field(default='')
    endpoint: str = Field(default='')
    bucket: str = Field(default='')


class MinioConfig(BaseSettings):
    model_config = SettingsConfigDict(env_file='.env', env_prefix='minio_', extra='ignore')

    access_key: str = Field(default='')
    secret_key: str = Field(default='')
    endpoint: str = Field(default='')
    bucket: str = Field(default='')
    region: str = Field(default='')
    secure: bool = Field(default=False)


class BrowserConfig(BaseSettings):
    model_config = SettingsConfigDict(env_file='.env', env_prefix='browser_', extra='ignore')

    headless: Optional[bool] = False
    simulate_device: Optional[Literal['iPhone 15', 'iPhone 15 Pro', 'iPhone 15 Pro Max', 'iPhone 6'] | str] = None


class OmniParserConfig(BaseSettings):
    model_config = SettingsConfigDict(env_file='.env', env_prefix='omni_', extra='ignore')

    base_url: Optional[str] = 'http://127.0.0.1:8000'
    key: Optional[str] = ''


class Settings(BaseSettings):
    model_config = SettingsConfigDict(env_file='.env', env_prefix='agent_', extra='ignore')

    root: Path = Field(default=Path(__file__).parent, exclude=True, repr=False)
    model: Optional[str] = 'openai:deepseek-chat'
    model_type: Optional[Literal['vlm', 'llm']] = 'llm'
    # max_tokens 限制模型输出的最大token数，数值越大，模型输出越长，越啰嗦，耗时也越长
    model_settings: ModelSettings = ModelSettings(max_tokens=500, temperature=0.2)

    browser: BrowserConfig = BrowserConfig()

    omni_parser: OmniParserConfig = OmniParserConfig()

    storage_client: StorageClient = StorageClient.create_from_config(CosConfig(), MinioConfig())

    debug: Optional[bool] = False


default_settings = Settings()
